@@ -12,13 +12,22 @@ from .cwt import (
     set_private_claim_names,
 )
 from .encrypted_cose_key import EncryptedCOSEKey
-from .enums import COSEType
+from .enums import (
+    COSEAlgs,
+    COSEHeaders,
+    COSEKeyCrvs,
+    COSEKeyOps,
+    COSEKeyParams,
+    COSEKeyTypes,
+    COSETypes,
+    CWTClaims,
+)
 from .exceptions import CWTError, DecodeError, EncodeError, VerifyError
 from .helpers.hcert import load_pem_hcert_dsc
 from .recipient import Recipient
 from .signer import Signer
 
-__version__ = "2.3.1"
+__version__ = "3.2.0"
 __title__ = "cwt"
 __description__ = "A Python implementation of CWT/COSE"
 __url__ = "https://python-cwt.readthedocs.io"
@@ -35,13 +44,19 @@ __all__ = [
     "encode_and_encrypt",
     "decode",
     "set_private_claim_names",
-    "CWT",
     "COSE",
+    "COSEAlgs",
     "COSEHeaders",
+    "COSEKeyCrvs",
+    "COSEKeyOps",
+    "COSEKeyParams",
+    "COSEKeyTypes",
+    "COSETypes",
     "COSEKey",
     "COSEMessage",
     "COSESignature",
-    "COSEType",
+    "CWT",
+    "CWTClaims",
     "EncryptedCOSEKey",
     "HPKECipherSuite",
     "Claims",
