@@ -1,23 +1,23 @@
-from .enums import COSEType
+from .enums import COSETypes
 
 COSE_TAG_TO_TYPE = {
-    16: COSEType.ENCRYPT0,
-    96: COSEType.ENCRYPT,
-    17: COSEType.MAC0,
-    97: COSEType.MAC,
-    18: COSEType.SIGN1,
-    98: COSEType.SIGN,
-    19: COSEType.COUNTERSIGNATURE,
+    16: COSETypes.ENCRYPT0,
+    96: COSETypes.ENCRYPT,
+    17: COSETypes.MAC0,
+    97: COSETypes.MAC,
+    18: COSETypes.SIGN1,
+    98: COSETypes.SIGN,
+    19: COSETypes.COUNTERSIGNATURE,
 }
 
 COSE_TYPE_TO_TAG = {
-    COSEType.ENCRYPT0: 16,
-    COSEType.ENCRYPT: 96,
-    COSEType.MAC0: 17,
-    COSEType.MAC: 97,
-    COSEType.SIGN1: 18,
-    COSEType.SIGN: 98,
-    COSEType.COUNTERSIGNATURE: 19,
+    COSETypes.ENCRYPT0: 16,
+    COSETypes.ENCRYPT: 96,
+    COSETypes.MAC0: 17,
+    COSETypes.MAC: 97,
+    COSETypes.SIGN1: 18,
+    COSETypes.SIGN: 98,
+    COSETypes.COUNTERSIGNATURE: 19,
 }
 
 # Registered CWT Claims
@@ -118,8 +118,9 @@ COSE_KEY_OPERATION_VALUES = {
     "verifyMAC": 10,  # JWK-like lowerCamelCase
 }
 
-# COSE Algorithms for Content Encryption Key (CEK).
-COSE_ALGORITHMS_CEK = {
+
+# COSE AEAD Algorithms
+COSE_ALGORITHMS_CEK_AEAD = {
     "A128GCM": 1,  # AES-GCM mode w/ 128-bit key, 128-bit tag
     "A192GCM": 2,  # AES-GCM mode w/ 192-bit key, 128-bit tag
     "A256GCM": 3,  # AES-GCM mode w/ 256-bit key, 128-bit tag
@@ -135,7 +136,29 @@ COSE_ALGORITHMS_CEK = {
     # etc.
 }
 
+# COSE non AEAD Algorithms defined in RFC9459
+COSE_ALGORITHMS_CEK_NON_AEAD = {
+    "A128CTR": -65534,  # AES-CTR mode w/ 128-bit key (Deprecated)
+    "A192CTR": -65533,  # AES-CTR mode w/ 192-bit key (Deprecated)
+    "A256CTR": -65532,  # AES-CTR mode w/ 256-bit key (Deprecated)
+    "A128CBC": -65531,  # AES-CBC mode w/ 128-bit key (Deprecated)
+    "A192CBC": -65530,  # AES-CBC mode w/ 192-bit key (Deprecated)
+    "A256CBC": -65529,  # AES-CBC mode w/ 256-bit key (Deprecated)
+}
+
+# COSE Algorithms for Content Encryption Key (CEK).
+COSE_ALGORITHMS_CEK = {
+    **COSE_ALGORITHMS_CEK_AEAD,
+    **COSE_ALGORITHMS_CEK_NON_AEAD,
+}
+
 COSE_KEY_LEN = {
+    -65534: 128,  # AES-CTR w/ 128-bit key (Deprecated)
+    -65533: 192,  # AES-CTR w/ 192-bit key (Deprecated)
+    -65532: 256,  # AES-CTR w/ 256-bit key (Deprecated)
+    -65531: 128,  # AES-CBC w/ 128-bit key (Deprecated)
+    -65530: 192,  # AES-CBC w/ 192-bit key (Deprecated)
+    -65529: 256,  # AES-CBC w/ 256-bit key (Deprecated)
     -5: 256,  # AES Key Wrap w/ 256-bit key
     -4: 192,  # AES Key Wrap w/ 192-bit key
     -3: 128,  # AES Key Wrap w/ 128-bit key
@@ -173,7 +196,16 @@ COSE_ALGORITHMS_KEY_WRAP = {
 }
 
 COSE_ALGORITHMS_HPKE = {
-    "HPKE": -1,  # HPKE
+    "HPKE-Base-P256-SHA256-AES128GCM": 35,
+    "HPKE-Base-P256-SHA256-ChaCha20Poly1305": 36,
+    "HPKE-Base-P384-SHA384-AES256GCM": 37,
+    "HPKE-Base-P384-SHA384-ChaCha20Poly1305": 38,
+    "HPKE-Base-P521-SHA512-AES256GCM": 39,
+    "HPKE-Base-P521-SHA512-ChaCha20Poly1305": 40,
+    "HPKE-Base-X448-SHA512-AES256GCM": 43,
+    "HPKE-Base-X448-SHA512-ChaCha20Poly1305": 44,
+    "HPKE-Base-X25519-SHA256-AES128GCM": 41,
+    "HPKE-Base-X25519-SHA256-ChaCha20Poly1305": 42,
 }
 
 COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_WITH_KEY_WRAP_SS = {
@@ -220,15 +252,20 @@ COSE_ALGORITHMS_MAC = {
 
 # COSE Algorithms for Signature with OKP.
 COSE_ALGORITHMS_SIG_OKP = {
-    "EdDSA": -8,
+    "EdDSA": -8,  # EdDSA
+    "Ed25519": -19,  # EdDSA using Ed25519 curve
+    "Ed448": -53,  # EdDSA using Ed448 curve
 }
 
 # COSE Algorithms for Signature with EC2.
 COSE_ALGORITHMS_SIG_EC2 = {
-    "ES256K": -47,
-    "ES512": -36,
-    "ES384": -35,
-    "ES256": -7,
+    "ESP512": -52,  # ECDSA using P-521 curve and SHA-512
+    "ESP384": -51,  # ECDSA using P-384 curve and SHA-384
+    "ES256K": -47,  # ECDSA using secp256k1 curve and SHA-256
+    "ES512": -36,  # ECDSA w/ SHA-512 (any curve is not specified but python-cwt uses P-521 curve)
+    "ES384": -35,  # ECDSA w/ SHA-384 (any curve is not specified but python-cwt uses P-384 curve)
+    "ESP256": -9,  # ECDSA using P-256 curve and SHA-256
+    "ES256": -7,  # ECDSA w/ SHA-256 (any curve is not specified but python-cwt uses P-256 curve)
 }
 
 # COSE Algorithms for Signature with RSA.
