@@ -1,11 +1,12 @@
 from typing import Any, Dict, List, Optional, Tuple, Union
 
-from asn1crypto import pem
 from cbor2 import CBORTag
+from cryptography.x509 import load_pem_x509_certificates
 
 from .cbor_processor import CBORProcessor
 from .const import (
     COSE_ALGORITHMS_CEK,
+    COSE_ALGORITHMS_CEK_NON_AEAD,
     COSE_ALGORITHMS_CKDM,
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT,
     COSE_ALGORITHMS_CKDM_KEY_AGREEMENT_DIRECT,
@@ -20,12 +21,12 @@ from .recipient_algs.hpke import HPKE
 from .recipient_interface import RecipientInterface
 from .recipients import Recipients
 from .signer import Signer
-from .utils import to_cose_header
+from .utils import ResolvedHeader, sort_keys_for_deterministic_encoding, to_cose_header
 
 
 class COSE(CBORProcessor):
     """
-    A COSE (CBOR Object Signing and Encryption) Implementaion built on top of
+    A COSE (CBOR Object Signing and Encryption) Implementation built on top of
     `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_.
     """
 
@@ -35,6 +36,7 @@ class COSE(CBORProcessor):
         kid_auto_inclusion: bool = False,
         verify_kid: bool = False,
         ca_certs: str = "",
+        deterministic_header: bool = False,
     ):
         if not isinstance(alg_auto_inclusion, bool):
             raise ValueError("alg_auto_inclusion should be bool.")
@@ -52,10 +54,11 @@ class COSE(CBORProcessor):
         if ca_certs:
             if not isinstance(ca_certs, str):
                 raise ValueError("ca_certs should be str.")
-            self._trust_roots: List[bytes] = []
             with open(ca_certs, "rb") as f:
-                for _, _, der_bytes in pem.unarmor(f.read(), multiple=True):
-                    self._ca_certs.append(der_bytes)
+                self._ca_certs = load_pem_x509_certificates(f.read())
+        if not isinstance(deterministic_header, bool):
+            raise ValueError("deterministic_header should be bool.")
+        self._deterministic_header = deterministic_header
 
     @classmethod
     def new(
@@ -64,6 +67,7 @@ class COSE(CBORProcessor):
         kid_auto_inclusion: bool = False,
         verify_kid: bool = False,
         ca_certs: str = "",
+        deterministic_header: bool = False,
     ):
         """
         Constructor.
@@ -79,8 +83,10 @@ class COSE(CBORProcessor):
                 of trusted root certificates. You should specify private CA
                 certificates in your target system. There should be no need to
                 use the public CA certificates for the Web PKI.
+            deterministic_header(bool): The indicator whether the protected and unprotected
+                headers will be deterministically encoded defined in section 4.2.1 of RFC 8949.
         """
-        return cls(alg_auto_inclusion, kid_auto_inclusion, verify_kid, ca_certs)
+        return cls(alg_auto_inclusion, kid_auto_inclusion, verify_kid, ca_certs, deterministic_header)
 
     @property
     def alg_auto_inclusion(self) -> bool:
@@ -126,12 +132,13 @@ class COSE(CBORProcessor):
         self,
         payload: bytes,
         key: Optional[COSEKeyInterface] = None,
-        protected: Optional[dict] = None,
-        unprotected: Optional[dict] = None,
+        protected: Optional[Union[dict, ResolvedHeader]] = None,
+        unprotected: Optional[Union[dict, ResolvedHeader]] = None,
         recipients: List[RecipientInterface] = [],
         signers: List[Signer] = [],
         external_aad: bytes = b"",
         out: str = "",
+        enable_non_aead: bool = False,
     ) -> bytes:
         """
         Encodes COSE message with MAC, signing and encryption.
@@ -139,8 +146,8 @@ class COSE(CBORProcessor):
         Args:
             payload (bytes): A content to be MACed, signed or encrypted.
             key (Optional[COSEKeyInterface]): A content encryption key as COSEKey.
-            protected (Optional[dict]): Parameters that are to be cryptographically protected.
-            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
+            protected (Optional[Union[dict, ResolvedHeader]]): Parameters that are to be cryptographically protected.
+            unprotected (Optional[Union[dict, ResolvedHeader]]): Parameters that are not cryptographically protected.
             recipients (List[RecipientInterface]): A list of recipient information structures.
             signers (List[Signer]): A list of signer information objects for
                 multiple signer cases.
@@ -151,6 +158,13 @@ class COSE(CBORProcessor):
                 data as `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s
                 ``CBORTag`` object. If any other value is specified, it will return
                 encoded data as bytes.
+            enable_non_aead (bool): Enable non-AEAD content ecnryption algorithms
+                (False = disabled by default). Before enable non-AEAD ciphers,
+                read and understand Security considerations of RFC 9459 carefully.
+                Since non-AEAD ciphers DO NOT provide neither authentication nor integrity
+                of decrypted message, make sure to deliver the encoded COSE message
+                in conjunction with an authentication and integrity mechanisms,
+                such as a digital signature.
         Returns:
             Union[bytes, CBORTag]: A byte string of the encoded COSE or a
                 cbor2.CBORTag object.
@@ -158,7 +172,7 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        p, u = self._encode_headers(key, protected, unprotected)
+        p, u = self._encode_headers(key, protected, unprotected, enable_non_aead)
         typ = self._validate_cose_message(key, p, u, recipients, signers)
         if typ == 0:
             return self._encode_and_encrypt(payload, key, p, u, recipients, external_aad, out)
@@ -176,6 +190,7 @@ class COSE(CBORProcessor):
         recipients: List[RecipientInterface] = [],
         external_aad: bytes = b"",
         out: str = "",
+        enable_non_aead: bool = False,
     ) -> bytes:
         """
         Encodes data with encryption.
@@ -193,6 +208,13 @@ class COSE(CBORProcessor):
                 data as `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s
                 ``CBORTag`` object. If any other value is specified, it will return
                 encoded data as bytes.
+            enable_non_aead (bool): Enable non-AEAD content ecnryption algorithms
+                (False = disabled by default). Before enable non-AEAD ciphers,
+                read and understand Security considerations of RFC 9459 carefully.
+                Since non-AEAD ciphers DO NOT provide neither authentication nor integrity
+                of decrypted message, make sure to deliver the encoded COSE message
+                in conjunction with an authentication and integrity mechanisms,
+                such as a digital signature.
         Returns:
             Union[bytes, CBORTag]: A byte string of the encoded COSE or a
                 cbor2.CBORTag object.
@@ -200,7 +222,7 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        p, u = self._encode_headers(key, protected, unprotected)
+        p, u = self._encode_headers(key, protected, unprotected, enable_non_aead)
         typ = self._validate_cose_message(key, p, u, recipients, [])
         if typ != 0:
             raise ValueError("The COSE message is not suitable for COSE Encrypt0/Encrypt.")
@@ -236,7 +258,7 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        p, u = self._encode_headers(key, protected, unprotected)
+        p, u = self._encode_headers(key, protected, unprotected, False)
         typ = self._validate_cose_message(key, p, u, recipients, [])
         if typ != 1:
             raise ValueError("The COSE message is not suitable for COSE MAC0/MAC.")
@@ -278,7 +300,7 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        p, u = self._encode_headers(key, protected, unprotected)
+        p, u = self._encode_headers(key, protected, unprotected, False)
         typ = self._validate_cose_message(key, p, u, [], signers)
         if typ != 2:
             raise ValueError("The COSE message is not suitable for COSE Sign0/Sign.")
@@ -290,9 +312,11 @@ class COSE(CBORProcessor):
         keys: Union[COSEKeyInterface, List[COSEKeyInterface]],
         context: Optional[Union[Dict[str, Any], List[Any]]] = None,
         external_aad: bytes = b"",
+        detached_payload: Optional[bytes] = None,
+        enable_non_aead: bool = False,
     ) -> bytes:
         """
-        Verifies and decodes COSE data.
+        Verifies and decodes COSE data, and returns only payload.
 
         Args:
             data (Union[bytes, CBORTag]): A byte string or cbor2.CBORTag of an
@@ -303,6 +327,12 @@ class COSE(CBORProcessor):
                 structure for key deriviation functions.
             external_aad(bytes): External additional authenticated data supplied by
                 application.
+            detached_payload (Optional[bytes]): The detached payload that should be verified with data.
+            enable_non_aead (bool): Enable non-AEAD content ecnryption algorithms
+                (False = disabled by default). Before enable non-AEAD ciphers,
+                read and understand Security considerations of RFC 9459 carefully.
+                Since non-AEAD ciphers DO NOT provide neither authentication nor integrity
+                of decrypted message, make sure to validate them outside of this library.
         Returns:
             bytes: A byte string of decoded payload.
         Raises:
@@ -310,6 +340,43 @@ class COSE(CBORProcessor):
             DecodeError: Failed to decode data.
             VerifyError: Failed to verify data.
         """
+        _, _, res = self.decode_with_headers(data, keys, context, external_aad, detached_payload, enable_non_aead)
+        return res
+
+    def decode_with_headers(
+        self,
+        data: Union[bytes, CBORTag],
+        keys: Union[COSEKeyInterface, List[COSEKeyInterface]],
+        context: Optional[Union[Dict[str, Any], List[Any]]] = None,
+        external_aad: bytes = b"",
+        detached_payload: Optional[bytes] = None,
+        enable_non_aead: bool = False,
+    ) -> Tuple[Dict[Union[str, int], Any], Dict[Union[str, int], Any], bytes]:
+        """
+        Verifies and decodes COSE data, and returns protected headers, unprotected headers and payload.
+
+        Args:
+            data (Union[bytes, CBORTag]): A byte string or cbor2.CBORTag of an
+                encoded data.
+            keys (Union[COSEKeyInterface, List[COSEKeyInterface]]): COSE key(s)
+                to verify and decrypt the encoded data.
+            context (Optional[Union[Dict[str, Any], List[Any]]]): A context information
+                structure for key deriviation functions.
+            external_aad(bytes): External additional authenticated data supplied by
+                application.
+            detached_payload (Optional[bytes]): The detached payload that should be verified with data.
+            enable_non_aead (bool): Enable non-AEAD content ecnryption algorithms
+                (False = disabled by default). Before enable non-AEAD ciphers,
+                read and understand Security considerations of RFC 9459 carefully.
+                Since non-AEAD ciphers DO NOT provide neither authentication nor integrity
+                of decrypted message, make sure to validate them outside of this library.
+        Returns:
+            Tuple[Dict[Union[str, int], Any], Dict[Union[str, int], Any], bytes]: A dictionary data of decoded protected headers, and a dictionary data of unprotected headers, and a byte string of decoded payload.
+        Raises:
+            ValueError: Invalid arguments.
+            DecodeError: Failed to decode data.
+            VerifyError: Failed to verify data.
+        """
         if isinstance(data, bytes):
             data = self._loads(data)
         if not isinstance(data, CBORTag):
@@ -347,38 +414,53 @@ class COSE(CBORProcessor):
         else:
             raise ValueError(f"Unsupported or unknown CBOR tag({data.tag}).")
 
+        payload = data.value[2]
+        if detached_payload is not None:
+            if data.value[2] is not None:
+                raise ValueError("The payload already exists.")
+            payload = detached_payload
+        if payload is None:
+            raise ValueError("detached_payload should be set.")
+
         # protected: Union[Dict[int, Any], bytes] = self._loads(data.value[0]) if data.value[0] else b""
         # unprotected = data.value[1]
         # if not isinstance(unprotected, dict):
         #     raise ValueError("unprotected header should be dict.")
         p, u = self._decode_headers(data.value[0], data.value[1])
-        alg = self._get_alg(p)
+        alg = p[1] if 1 in p else u.get(1, 0)
+        if enable_non_aead is False and alg in COSE_ALGORITHMS_CEK_NON_AEAD.values():
+            raise ValueError(f"Deprecated non-AEAD algorithm: {alg}.")
+
+        # Local variable `protected` is byte encoded protected header
+        # Sender is allowed to encode empty protected header into a bstr-wrapped zero-length map << {} >> (0x40A0)
+        # but Recipient MUST treat it as a zero-length byte string h'' (0x40) while decoding
+        protected = data.value[0] if len(p) > 0 else b""
 
         err: Exception = ValueError("key is not found.")
 
         # Encrypt0
         if data.tag == 16:
             kid = self._get_kid(p, u)
-            aad = self._dumps(["Encrypt0", data.value[0], external_aad])
+            aad = self._dumps(["Encrypt0", protected, external_aad])
             nonce = u.get(5, None)
             if kid:
                 for _, k in enumerate(keys):
                     if k.kid != kid:
                         continue
                     try:
-                        if not isinstance(p, bytes) and alg == -1:  # HPKE
-                            hpke = HPKE(p, u, data.value[2])
+                        if not isinstance(p, bytes) and alg in COSE_ALGORITHMS_HPKE.values():  # HPKE
+                            hpke = HPKE(p, u, payload)
                             res = hpke.decode(k, aad)
                             if not isinstance(res, bytes):
                                 raise TypeError("Internal type error.")
-                            return res
-                        return k.decrypt(data.value[2], nonce, aad)
+                            return p, u, res
+                        return p, u, k.decrypt(payload, nonce, aad)
                     except Exception as e:
                         err = e
                 raise err
             for _, k in enumerate(keys):
                 try:
-                    return k.decrypt(data.value[2], nonce, aad)
+                    return p, u, k.decrypt(payload, nonce, aad)
                 except Exception as e:
                     err = e
             raise err
@@ -389,42 +471,42 @@ class COSE(CBORProcessor):
             nonce = u.get(5, b"")
             enc_key = rs.derive_key(keys, alg, external_aad, "Enc_Recipient")
             aad = self._dumps(["Encrypt", data.value[0], external_aad])
-            return enc_key.decrypt(data.value[2], nonce, aad)
+            return p, u, enc_key.decrypt(payload, nonce, aad)
 
         # MAC0
         if data.tag == 17:
             kid = self._get_kid(p, u)
-            msg = self._dumps(["MAC0", data.value[0], external_aad, data.value[2]])
+            msg = self._dumps(["MAC0", protected, external_aad, payload])
             if kid:
                 for _, k in enumerate(keys):
                     if k.kid != kid:
                         continue
                     try:
                         k.verify(msg, data.value[3])
-                        return data.value[2]
+                        return p, u, payload
                     except Exception as e:
                         err = e
                 raise err
             for _, k in enumerate(keys):
                 try:
                     k.verify(msg, data.value[3])
-                    return data.value[2]
+                    return p, u, payload
                 except Exception as e:
                     err = e
             raise err
 
         # MAC
         if data.tag == 97:
-            to_be_maced = self._dumps(["MAC", data.value[0], external_aad, data.value[2]])
+            to_be_maced = self._dumps(["MAC", protected, external_aad, payload])
             rs = Recipients.from_list(data.value[4], self._verify_kid, context)
             mac_auth_key = rs.derive_key(keys, alg, external_aad, "Mac_Recipient")
             mac_auth_key.verify(to_be_maced, data.value[3])
-            return data.value[2]
+            return p, u, payload
 
         # Signature1
         if data.tag == 18:
             kid = self._get_kid(p, u)
-            to_be_signed = self._dumps(["Signature1", data.value[0], external_aad, data.value[2]])
+            to_be_signed = self._dumps(["Signature1", protected, external_aad, payload])
             if kid:
                 for _, k in enumerate(keys):
                     if k.kid != kid:
@@ -433,7 +515,7 @@ class COSE(CBORProcessor):
                         if self._ca_certs:
                             k.validate_certificate(self._ca_certs)
                         k.verify(to_be_signed, data.value[3])
-                        return data.value[2]
+                        return p, u, payload
                     except Exception as e:
                         err = e
                 raise err
@@ -442,7 +524,7 @@ class COSE(CBORProcessor):
                     if self._ca_certs:
                         k.validate_certificate(self._ca_certs)
                     k.verify(to_be_signed, data.value[3])
-                    return data.value[2]
+                    return p, u, payload
                 except Exception as e:
                     err = e
             raise err
@@ -469,14 +551,14 @@ class COSE(CBORProcessor):
                         to_be_signed = self._dumps(
                             [
                                 "Signature",
-                                data.value[0],
+                                protected,
                                 sig[0],
                                 external_aad,
-                                data.value[2],
+                                payload,
                             ]
                         )
                         k.verify(to_be_signed, sig[2])
-                        return data.value[2]
+                        return p, u, payload
                     except Exception as e:
                         err = e
                 continue
@@ -485,14 +567,14 @@ class COSE(CBORProcessor):
                     to_be_signed = self._dumps(
                         [
                             "Signature",
-                            data.value[0],
+                            protected,
                             sig[0],
                             external_aad,
-                            data.value[2],
+                            payload,
                         ]
                     )
                     k.verify(to_be_signed, sig[2])
-                    return data.value[2]
+                    return p, u, payload
                 except Exception as e:
                     err = e
         raise err
@@ -500,26 +582,46 @@ class COSE(CBORProcessor):
     def _encode_headers(
         self,
         key: Optional[COSEKeyInterface],
-        protected: Optional[dict],
-        unprotected: Optional[dict],
-    ) -> Tuple[Dict[int, Any], Dict[int, Any]]:
+        protected: Optional[Union[dict, ResolvedHeader]],
+        unprotected: Optional[Union[dict, ResolvedHeader]],
+        enable_non_aead: bool,
+    ) -> Tuple[Dict[Union[str, int], Any], Dict[Union[str, int], Any]]:
         p = to_cose_header(protected)
         u = to_cose_header(unprotected)
         if key is not None:
             if self._alg_auto_inclusion:
-                p[1] = key.alg
+                if key.alg in COSE_ALGORITHMS_CEK_NON_AEAD.values():
+                    u[1] = key.alg
+                else:
+                    p[1] = key.alg
             if self._kid_auto_inclusion and key.kid:
                 u[4] = key.kid
+
+        # sort the key for deterministic encoding
+        if self._deterministic_header:
+            p = sort_keys_for_deterministic_encoding(p)
+            u = sort_keys_for_deterministic_encoding(u)
+
+        # Check the protected header is empty if the algorithm is non AEAD (AES-CBC or AES-CTR)
+        # because section 4 of RFC9459 says "The 'protected' header MUST be a zero-length byte string."
+        alg = p[1] if 1 in p else u.get(1, 0)
+        if alg in COSE_ALGORITHMS_CEK_NON_AEAD.values():
+            if enable_non_aead is False:
+                raise ValueError(f"Deprecated non-AEAD algorithm: {alg}.")
+            if len(p) > 0:
+                raise ValueError("protected header MUST be zero-length")
         return p, u
 
-    def _decode_headers(self, protected: Any, unprotected: Any) -> Tuple[Dict[int, Any], Dict[int, Any]]:
-        p: Union[Dict[int, Any], bytes]
+    def _decode_headers(
+        self, protected: Any, unprotected: Any
+    ) -> Tuple[Dict[Union[str, int], Any], Dict[Union[str, int], Any]]:
+        p: Union[Dict[Union[str, int], Any], bytes]
         p = self._loads(protected) if protected else {}
         if isinstance(p, bytes):
             if len(p) > 0:
                 raise ValueError("Invalid protected header.")
             p = {}
-        u: Dict[int, Any] = unprotected
+        u: Dict[Union[str, int], Any] = unprotected
         if not isinstance(u, dict):
             raise ValueError("unprotected header should be dict.")
         return p, u
@@ -527,15 +629,15 @@ class COSE(CBORProcessor):
     def _validate_cose_message(
         self,
         key: Optional[COSEKeyInterface],
-        p: Dict[int, Any],
-        u: Dict[int, Any],
+        p: Dict[Union[str, int], Any],
+        u: Dict[Union[str, int], Any],
         recipients: List[RecipientInterface],
         signers: List[Signer],
     ) -> int:
         if len(recipients) > 0 and len(signers) > 0:
             raise ValueError("Both recipients and signers are specified.")
 
-        h: Dict[int, Any] = {}
+        h: Dict[Union[str, int], Any] = {}
         iv_count: int = 0
         for k, v in p.items():
             if k == 2:  # crit
@@ -633,6 +735,8 @@ class COSE(CBORProcessor):
                 return 0  # Encrypt
             if recipients[0].context[0] in COSE_ALGORITHMS_HPKE.values():
                 return 0  # Encrypt
+            if recipients[0].context[0] in COSE_ALGORITHMS_KEY_WRAP.values():
+                return 0  # Encrypt
             if recipients[0].context[0] in COSE_ALGORITHMS_MAC.values():
                 return 1  # MAC
             raise ValueError(f"Invalid alg in recipients' context information: {recipients[0]._context[0]}.")
@@ -643,8 +747,8 @@ class COSE(CBORProcessor):
         self,
         payload: bytes,
         key: Optional[COSEKeyInterface],
-        p: Dict[int, Any],
-        u: Dict[int, Any],
+        p: Dict[Union[str, int], Any],
+        u: Dict[Union[str, int], Any],
         recipients: List[RecipientInterface],
         external_aad: bytes,
         out: str,
@@ -656,7 +760,7 @@ class COSE(CBORProcessor):
         if len(recipients) == 0:
             enc_structure = ["Encrypt0", b_protected, external_aad]
             aad = self._dumps(enc_structure)
-            if 1 in p and p[1] == -1:  # HPKE
+            if 1 in p and p[1] in COSE_ALGORITHMS_HPKE.values():  # HPKE
                 hpke = HPKE(p, u, recipient_key=key)
                 encoded, _ = hpke.encode(payload, aad)
                 res = CBORTag(16, encoded)
@@ -680,7 +784,7 @@ class COSE(CBORProcessor):
         b_key = key.to_bytes() if isinstance(key, COSEKeyInterface) else b""
         cek: Optional[COSEKeyInterface] = None
         for rec in recipients:
-            aad = self._dumps(["Enc_Recipient", self._dumps(rec.protected), external_aad])
+            aad = self._dumps(["Enc_Recipient", self._dumps(rec.protected) if len(rec.protected) > 0 else b"", external_aad])
             encoded, derived_key = rec.encode(b_key, aad)
             cek = derived_key if derived_key else key
             recs.append(encoded)
@@ -704,8 +808,8 @@ class COSE(CBORProcessor):
         self,
         payload: bytes,
         key: Optional[COSEKeyInterface],
-        p: Dict[int, Any],
-        u: Dict[int, Any],
+        p: Dict[Union[str, int], Any],
+        u: Dict[Union[str, int], Any],
         recipients: List[RecipientInterface],
         external_aad: bytes,
         out: str,
@@ -747,8 +851,8 @@ class COSE(CBORProcessor):
         self,
         payload: bytes,
         key: Optional[COSEKeyInterface],
-        p: Dict[int, Any],
-        u: Dict[int, Any],
+        p: Dict[Union[str, int], Any],
+        u: Dict[Union[str, int], Any],
         signers: List[Signer],
         external_aad: bytes,
         out: str,
