@@ -316,7 +316,7 @@ class CWT(CBORProcessor):
         data: bytes,
         keys: Union[COSEKeyInterface, List[COSEKeyInterface]],
         no_verify: bool = False,
-    ) -> Union[Dict[int, Any], bytes]:
+    ) -> Union[Dict[Union[str, int], Any], bytes]:
         """
         Verifies and decodes CWT.
 
@@ -333,15 +333,16 @@ class CWT(CBORProcessor):
             DecodeError: Failed to decode the CWT.
             VerifyError: Failed to verify the CWT.
         """
-        cwt: Union[bytes, CBORTag, Dict[int, Any]] = self._loads(data)
+        cwt: Union[bytes, CBORTag, Dict[Union[str, int], Any]] = self._loads(data)
         if isinstance(cwt, CBORTag) and cwt.tag == CWT.CBOR_TAG:
             cwt = cwt.value
         keys = [keys] if isinstance(keys, COSEKeyInterface) else keys
+        p: Dict[Union[str, int], Any] = {}
         while isinstance(cwt, CBORTag):
-            cwt = self._cose.decode(cwt, keys)
+            p, u, cwt = self._cose.decode_with_headers(cwt, keys)
             cwt = self._loads(cwt)
         if not no_verify:
-            self._verify(cwt)
+            self._verify(cwt, p)
         return cwt
 
     def set_private_claim_names(self, claim_names: Dict[str, int]):
@@ -398,7 +399,7 @@ class CWT(CBORProcessor):
         Claims.validate(claims)
         return
 
-    def _verify(self, claims: Union[Dict[int, Any], bytes]):
+    def _verify(self, claims: Union[Dict[Union[str, int], Any], bytes], protected: Dict[Union[str, int], Any] = {}):
         if not isinstance(claims, dict):
             raise DecodeError("Failed to decode.")
 
@@ -416,6 +417,11 @@ class CWT(CBORProcessor):
                     raise VerifyError("The token is not yet valid.")
             else:
                 raise ValueError("nbf should be int or float.")
+
+        if 13 in protected:  # CWT claims in protected headers
+            for k, v in protected[13].items():
+                if k in claims and claims[k] != v:
+                    raise VerifyError(f"The CWT claim({k}) value in protected header does not match the values in the payload.")
         return
 
     def _set_default_value(self, claims: Union[Dict[int, Any], bytes]):
@@ -478,7 +484,7 @@ def decode(
     data: bytes,
     keys: Union[COSEKeyInterface, List[COSEKeyInterface]],
     no_verify: bool = False,
-) -> Union[Dict[int, Any], bytes]:
+) -> Union[Dict[Union[str, int], Any], bytes]:
     return _cwt.decode(data, keys, no_verify)
 
 
