@@ -59,7 +59,7 @@ class SwedishVerifier:
             json.dump(self._trustlist, f, indent=4)
         return
 
-    def verify_and_decode(self, eudcc: bytes) -> Union[Dict[int, Any], bytes]:
+    def verify_and_decode(self, eudcc: bytes) -> Union[Dict[Union[str, int], Any], bytes]:
         if eudcc.startswith(b"HC1:"):
             # Decode Base45 data.
             eudcc = b45decode(eudcc[4:])
@@ -74,7 +74,7 @@ class SwedishVerifier:
                 self._trustlist = json.load(f)
             self._dscs = [COSEKey.from_jwk(k) for k in self._trustlist]
         except Exception as err:
-            if type(err) != FileNotFoundError:
+            if type(err) is not FileNotFoundError:
                 raise err
             self._trustlist = []
         return
