@@ -9,7 +9,7 @@ Tests for COSE.
 
 import pytest
 
-from cwt import COSE, COSEKey
+from cwt import COSE, COSEAlgs, COSEHeaders, COSEKey
 
 
 class TestCOSE_HPKE:
@@ -18,20 +18,13 @@ class TestCOSE_HPKE:
     """
 
     @pytest.mark.parametrize(
-        "kdf, aead",
+        "alg",
         [
-            (0x0001, 0x0001),
-            (0x0001, 0x0002),
-            (0x0001, 0x0003),
-            (0x0002, 0x0001),
-            (0x0002, 0x0002),
-            (0x0002, 0x0003),
-            (0x0003, 0x0001),
-            (0x0003, 0x0002),
-            (0x0003, 0x0003),
+            COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
+            COSEAlgs.HPKE_BASE_P256_SHA256_CHACHA20POLY1305,
         ],
     )
-    def test_cose_hpke_kem_0x0010(self, kdf, aead):
+    def test_cose_hpke_kem_0x0010(self, alg):
         rpk = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -47,15 +40,10 @@ class TestCOSE_HPKE:
             b"This is the content.",
             rpk,
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: alg,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,
-                    kdf,
-                    aead,
-                ],
+                COSEHeaders.KID: b"01",  # kid: "01"
             },
         )
 
@@ -74,20 +62,13 @@ class TestCOSE_HPKE:
         assert b"This is the content." == recipient.decode(encoded, rsk)
 
     @pytest.mark.parametrize(
-        "kdf, aead",
+        "alg",
         [
-            (0x0001, 0x0001),
-            (0x0001, 0x0002),
-            (0x0001, 0x0003),
-            (0x0002, 0x0001),
-            (0x0002, 0x0002),
-            (0x0002, 0x0003),
-            (0x0003, 0x0001),
-            (0x0003, 0x0002),
-            (0x0003, 0x0003),
+            COSEAlgs.HPKE_BASE_P384_SHA384_AES256GCM,
+            COSEAlgs.HPKE_BASE_P384_SHA384_CHACHA20POLY1305,
         ],
     )
-    def test_cose_hpke_kem_0x0011(self, kdf, aead):
+    def test_cose_hpke_kem_0x0011(self, alg):
         rpk = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -103,15 +84,10 @@ class TestCOSE_HPKE:
             b"This is the content.",
             rpk,
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: alg,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0011,
-                    kdf,
-                    aead,
-                ],
+                COSEHeaders.KID: b"01",  # kid: "01"
             },
         )
 
@@ -130,20 +106,13 @@ class TestCOSE_HPKE:
         assert b"This is the content." == recipient.decode(encoded, rsk)
 
     @pytest.mark.parametrize(
-        "kdf, aead",
+        "alg",
         [
-            (0x0001, 0x0001),
-            (0x0001, 0x0002),
-            (0x0001, 0x0003),
-            (0x0002, 0x0001),
-            (0x0002, 0x0002),
-            (0x0002, 0x0003),
-            (0x0003, 0x0001),
-            (0x0003, 0x0002),
-            (0x0003, 0x0003),
+            COSEAlgs.HPKE_BASE_P521_SHA512_AES256GCM,
+            COSEAlgs.HPKE_BASE_P521_SHA512_CHACHA20POLY1305,
         ],
     )
-    def test_cose_hpke_kem_0x0012(self, kdf, aead):
+    def test_cose_hpke_kem_0x0012(self, alg):
         rpk = COSEKey.from_jwk(
             {
                 "kty": "EC",
@@ -159,15 +128,10 @@ class TestCOSE_HPKE:
             b"This is the content.",
             rpk,
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: alg,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0012,
-                    kdf,
-                    aead,
-                ],
+                COSEHeaders.KID: b"01",  # kid: "01"
             },
         )
 
@@ -186,28 +150,20 @@ class TestCOSE_HPKE:
         assert b"This is the content." == recipient.decode(encoded, rsk)
 
     @pytest.mark.parametrize(
-        "kdf, aead",
+        "alg",
         [
-            (0x0001, 0x0001),
-            (0x0001, 0x0002),
-            (0x0001, 0x0003),
-            (0x0002, 0x0001),
-            (0x0002, 0x0002),
-            (0x0002, 0x0003),
-            (0x0003, 0x0001),
-            (0x0003, 0x0002),
-            (0x0003, 0x0003),
+            COSEAlgs.HPKE_BASE_X25519_SHA256_AES128GCM,
+            COSEAlgs.HPKE_BASE_X25519_SHA256_CHACHA20POLY1305,
         ],
     )
-    def test_cose_hpke_kem_0x0020(self, kdf, aead):
+    def test_cose_hpke_kem_0x0020(self, alg):
         rpk = COSEKey.from_jwk(
             {
                 "kty": "OKP",
                 "crv": "X25519",
                 "kid": "01",
-                "alg": "HPKE",
                 "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
-                "key_ops": ["deriveKey", "deriveBits"],
+                "key_ops": [],
             }
         )
 
@@ -216,15 +172,10 @@ class TestCOSE_HPKE:
             b"This is the content.",
             rpk,
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: alg,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0020,
-                    kdf,
-                    aead,
-                ],
+                COSEHeaders.KID: b"01",  # kid: "01"
             },
         )
 
@@ -234,38 +185,28 @@ class TestCOSE_HPKE:
                 "kty": "OKP",
                 "crv": "X25519",
                 "kid": "01",
-                "alg": "HPKE",
                 "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
                 "d": "vsJ1oX5NNi0IGdwGldiac75r-Utmq3Jq4LGv48Q_Qc4",
-                "key_ops": ["deriveKey", "deriveBits"],
+                "key_ops": ["deriveBits"],
             }
         )
         recipient = COSE.new()
         assert b"This is the content." == recipient.decode(encoded, rsk)
 
     @pytest.mark.parametrize(
-        "kdf, aead",
+        "alg",
         [
-            (0x0001, 0x0001),
-            (0x0001, 0x0002),
-            (0x0001, 0x0003),
-            (0x0002, 0x0001),
-            (0x0002, 0x0002),
-            (0x0002, 0x0003),
-            (0x0003, 0x0001),
-            (0x0003, 0x0002),
-            (0x0003, 0x0003),
+            COSEAlgs.HPKE_BASE_X448_SHA512_AES256GCM,
+            COSEAlgs.HPKE_BASE_X448_SHA512_CHACHA20POLY1305,
         ],
     )
-    def test_cose_hpke_kem_0x0021(self, kdf, aead):
+    def test_cose_hpke_kem_0x0021(self, alg):
         rpk = COSEKey.from_jwk(
             {
                 "kty": "OKP",
                 "crv": "X448",
                 "kid": "01",
-                "alg": "HPKE",
                 "x": "IkLmc0klvEMXYneHMKAB6ePohryAwAPVe2pRSffIDY6NrjeYNWVX5J-fG4NV2OoU77C88A0mvxI",
-                "key_ops": ["deriveKey"],
             }
         )
 
@@ -274,15 +215,10 @@ class TestCOSE_HPKE:
             b"This is the content.",
             rpk,
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: alg,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0021,
-                    kdf,
-                    aead,
-                ],
+                COSEHeaders.KID: b"01",  # kid: "01"
             },
         )
 
@@ -292,10 +228,9 @@ class TestCOSE_HPKE:
                 "kty": "OKP",
                 "crv": "X448",
                 "kid": "01",
-                "alg": "HPKE",
                 "x": "IkLmc0klvEMXYneHMKAB6ePohryAwAPVe2pRSffIDY6NrjeYNWVX5J-fG4NV2OoU77C88A0mvxI",
                 "d": "rJJRG3nshyCtd9CgXld8aNaB9YXKR0UOi7zj7hApg9YH4XdBO0G8NcAFNz_uPH2GnCZVcSDgV5c",
-                "key_ops": ["deriveKey"],
+                "key_ops": ["deriveBits"],
             }
         )
         recipient = COSE.new()
