@@ -2,7 +2,7 @@ from secrets import token_bytes
 
 import pytest
 
-from cwt import COSE, COSEKey, Recipient, Signer
+from cwt import COSE, COSEAlgs, COSEHeaders, COSEKey, Recipient, Signer, utils
 
 
 class TestCOSESample:
@@ -34,8 +34,8 @@ class TestCOSESample:
         encoded3 = sender.encode_and_mac(
             b"Hello world!",
             mac_key,
-            protected={1: 5},
-            unprotected={4: b"01"},
+            protected={COSEHeaders.ALG: COSEAlgs.HS256},
+            unprotected={COSEHeaders.KID: b"01"},
         )
         assert b"Hello world!" == recipient.decode(encoded3, mac_key)
 
@@ -71,12 +71,12 @@ class TestCOSESample:
         )
         assert b"Hello world!" == recipient.decode(encoded2, mac_key)
 
-        r3 = Recipient.new(unprotected={1: -6})
+        r3 = Recipient.new(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT})
         encoded3 = sender.encode_and_mac(
             b"Hello world!",
             mac_key,
-            protected={1: 7},
-            unprotected={4: mac_key.kid},
+            protected={COSEHeaders.ALG: COSEAlgs.HS512},
+            unprotected={COSEHeaders.KID: mac_key.kid},
             recipients=[r3],
         )
         assert b"Hello world!" == recipient.decode(encoded3, mac_key)
@@ -267,7 +267,7 @@ class TestCOSESample:
         # The sender side:
         nonce = enc_key.generate_nonce()
         sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
-        encoded = sender.encode_and_encrypt(b"Hello world!", enc_key, unprotected={5: nonce})
+        encoded = sender.encode_and_encrypt(b"Hello world!", enc_key, unprotected={COSEHeaders.IV: nonce})
 
         # The recipient side:
         recipient = COSE.new()
@@ -286,8 +286,8 @@ class TestCOSESample:
         encoded3 = sender.encode_and_encrypt(
             b"Hello world!",
             enc_key,
-            protected={1: 24},
-            unprotected={4: b"01", 5: nonce},
+            protected={COSEHeaders.ALG: COSEAlgs.CHACHA20_POLY1305},
+            unprotected={COSEHeaders.KID: b"01", COSEHeaders.IV: nonce},
         )
         assert b"Hello world!" == recipient.decode(encoded3, enc_key)
 
@@ -300,7 +300,6 @@ class TestCOSESample:
                 "kty": "EC",
                 "kid": "01",
                 "crv": "P-256",
-                # "alg": "HPKE",
                 "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
                 "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
             }
@@ -311,15 +310,10 @@ class TestCOSESample:
             b"This is the content.",
             rpk,
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
         )
 
@@ -361,12 +355,12 @@ class TestCOSESample:
         assert b"Hello world!" == recipient.decode(encoded, enc_key)
 
         # variation
-        r = Recipient.new(unprotected={1: -6})
+        r = Recipient.new(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT})
         encoded2 = sender.encode_and_encrypt(
             b"Hello world!",
             enc_key,
-            protected={1: 24},  # ChaCha20/Poly1305
-            unprotected={4: enc_key.kid, 5: nonce},
+            protected={COSEHeaders.ALG: COSEAlgs.CHACHA20_POLY1305},
+            unprotected={COSEHeaders.KID: enc_key.kid, COSEHeaders.IV: nonce},
             recipients=[r],
         )
         assert b"Hello world!" == recipient.decode(encoded2, enc_key)
@@ -387,15 +381,10 @@ class TestCOSESample:
         )
         r = Recipient.new(
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
             recipient_key=rpk,
         )
@@ -436,15 +425,10 @@ class TestCOSESample:
         )
         r = Recipient.new(
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
             recipient_key=rpk,
         )
@@ -453,15 +437,10 @@ class TestCOSESample:
             sender.encode_and_encrypt(
                 b"This is the content.",
                 protected={
-                    1: -1,  # alg: "HPKE"
+                    COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
                 },
                 unprotected={
-                    4: b"xx",  # kid: "xx"
-                    -4: [  # HPKE sender information
-                        0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                        0x0001,  # kdf: HKDF-SHA256
-                        0x0001,  # aead: AES-128-GCM
-                    ],
+                    COSEHeaders.KID: b"xx",
                 },
                 recipients=[r],
             )
@@ -482,15 +461,10 @@ class TestCOSESample:
         )
         r = Recipient.new(
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
             recipient_key=rpk,
         )
@@ -499,15 +473,10 @@ class TestCOSESample:
             sender.encode_and_encrypt(
                 b"This is the content.",
                 protected={
-                    1: -1,  # alg: "HPKE"
+                    COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
                 },
                 unprotected={
-                    4: b"xx",  # kid: "xx"
-                    -4: [  # HPKE sender information
-                        0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                        0x0001,  # kdf: HKDF-SHA256
-                        0x0001,  # aead: AES-128-GCM
-                    ],
+                    COSEHeaders.KID: b"xx",  # kid: "xx"
                 },
                 recipients=[r],
             )
@@ -631,7 +600,7 @@ class TestCOSESample:
         encoded = sender.encode_and_encrypt(
             b"Hello world!",
             key=enc_key,
-            unprotected={5: nonce},
+            unprotected={COSEHeaders.IV: nonce},
             recipients=[r],
         )
 
@@ -691,8 +660,8 @@ class TestCOSESample:
         encoded3 = sender.encode_and_sign(
             b"Hello world!",
             priv_key,
-            protected={1: -7},
-            unprotected={4: b"01"},
+            protected={COSEHeaders.ALG: COSEAlgs.ES256},
+            unprotected={COSEHeaders.KID: b"01"},
         )
         assert b"Hello world!" == recipient.decode(encoded3, pub_key)
 
@@ -794,8 +763,64 @@ class TestCOSESample:
                     "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
                 }
             ),
-            protected={1: -7},
-            unprotected={4: b"01"},
+            protected={COSEHeaders.ALG: COSEAlgs.ES256},
+            unprotected={COSEHeaders.KID: b"01"},
         )
         encoded3 = sender.encode_and_sign(b"Hello world!", signers=[signer])
         assert b"Hello world!" == recipient.decode(encoded3, pub_key)
+
+    def test_cose_usage_with_resolved_header(self):
+        cose_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            }
+        )
+
+        recipient = COSE.new()
+        pub_key = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+
+        ctx = COSE.new(alg_auto_inclusion=True)
+
+        payload = b"Hello world!"
+        protected_first_not_str = {COSEHeaders.KID: b"01", COSEHeaders.ALG: COSEAlgs.ES256, "string key": "value"}
+
+        # This works because the first key is not a string, and so the early exit in to_cose_header() happens
+        encoded = ctx.encode_and_sign(payload, cose_key, protected=protected_first_not_str)
+        phdr, _, payload_ = recipient.decode_with_headers(encoded, pub_key)
+        assert payload_ == payload
+        assert phdr[COSEHeaders.ALG] == COSEAlgs.ES256
+        assert phdr["string key"] == "value"
+
+        # This works because ResolvedHeader always skips string key encoding
+        encoded = ctx.encode_and_sign(payload, cose_key, protected=utils.ResolvedHeader(protected_first_not_str))
+        phdr, _, payload_ = recipient.decode_with_headers(encoded, pub_key)
+        assert payload_ == payload
+        assert phdr[COSEHeaders.ALG] == COSEAlgs.ES256
+        assert phdr["string key"] == "value"
+
+        protected_first_str = {"string key": "value", COSEHeaders.KID: b"01", COSEHeaders.ALG: COSEAlgs.ES256}
+
+        with pytest.raises(ValueError):
+            # Raises ValueError: Unsupported or unknown COSE header parameter: 4.
+            # This fails because the first key is a string, and to_cose_header attempts to resolve the params
+            ctx.encode_and_sign(b"Hello world!", cose_key, protected=protected_first_str)
+
+        # This works because ResolvedHeader always skips string key encoding
+        encoded = ctx.encode_and_sign(payload, cose_key, protected=utils.ResolvedHeader(protected_first_str))
+        phdr, _, payload_ = recipient.decode_with_headers(encoded, pub_key)
+        assert payload_ == payload
+        assert phdr[COSEHeaders.ALG] == COSEAlgs.ES256
+        assert phdr["string key"] == "value"
