@@ -1,8 +1,9 @@
 from secrets import token_bytes
 
 import pytest
+from cbor2 import dumps, loads
 
-from cwt import COSE, COSEKey, COSEMessage, Recipient, Signer
+from cwt import COSE, COSEAlgs, COSEHeaders, COSEKey, COSEMessage, Recipient, Signer
 
 
 class TestCOSESampleWithEncode:
@@ -34,8 +35,8 @@ class TestCOSESampleWithEncode:
         encoded3 = sender.encode(
             b"Hello world!",
             mac_key,
-            protected={1: 5},
-            unprotected={4: b"01"},
+            protected={COSEHeaders.ALG: COSEAlgs.HS256},
+            unprotected={COSEHeaders.KID: b"01"},
         )
         assert b"Hello world!" == recipient.decode(encoded3, mac_key)
 
@@ -99,7 +100,7 @@ class TestCOSESampleWithEncode:
         encoded2 = sender.encode(b"Hello world!", mac_key, protected={"alg": "HS512"}, recipients=[r2])
         assert b"Hello world!" == recipient.decode(encoded2, mac_key)
 
-        r3 = Recipient.new(unprotected={1: -6, 4: mac_key.kid})
+        r3 = Recipient.new(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: mac_key.kid})
         encoded3 = sender.encode(b"Hello world!", mac_key, protected={"alg": "HS512"}, recipients=[r3])
         assert b"Hello world!" == recipient.decode(encoded3, mac_key)
 
@@ -327,7 +328,7 @@ class TestCOSESampleWithEncode:
         mac_key = COSEKey.generate_symmetric_key(alg="HS256")
 
         # The sender side:
-        rpk = COSEKey.from_jwk(
+        rpk1 = COSEKey.from_jwk(
             {
                 "kty": "EC",
                 "kid": "01",
@@ -336,32 +337,45 @@ class TestCOSESampleWithEncode:
                 "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
             }
         )
-        r = Recipient.new(
+        r1 = Recipient.new(
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
-            recipient_key=rpk,
+            recipient_key=rpk1,
+        )
+        rpk2 = COSEKey.from_jwk(
+            {
+                "kty": "OKP",
+                "kid": "02",
+                "crv": "X25519",
+                "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
+            },
+        )
+        r2 = Recipient.new(
+            protected={
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_X25519_SHA256_CHACHA20POLY1305,
+            },
+            unprotected={
+                COSEHeaders.KID: b"02",
+            },
+            recipient_key=rpk2,
         )
         sender = COSE.new()
         encoded = sender.encode(
             b"This is the content.",
             mac_key,
-            protected={1: 5},  # alg: HS256
-            recipients=[r],
+            protected={COSEHeaders.ALG: COSEAlgs.HS256},  # alg: HS256
+            recipients=[r1, r2],
+            external_aad=b"COSE-HPKE app",
         )
 
         # print(encoded.hex())
 
         # The recipient side:
-        rsk = COSEKey.from_jwk(
+        rsk1 = COSEKey.from_jwk(
             {
                 "kty": "EC",
                 "kid": "01",
@@ -372,7 +386,18 @@ class TestCOSESampleWithEncode:
             }
         )
         recipient = COSE.new()
-        assert b"This is the content." == recipient.decode(encoded, rsk)
+        assert b"This is the content." == recipient.decode(encoded, rsk1, external_aad=b"COSE-HPKE app")
+
+        rsk2 = COSEKey.from_jwk(
+            {
+                "kty": "OKP",
+                "kid": "02",
+                "crv": "X25519",
+                "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
+                "d": "vsJ1oX5NNi0IGdwGldiac75r-Utmq3Jq4LGv48Q_Qc4",
+            },
+        )
+        assert b"This is the content." == recipient.decode(encoded, rsk2, external_aad=b"COSE-HPKE app")
 
     def test_cose_usage_examples_cose_encrypt0(self):
         enc_key = COSEKey.generate_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
@@ -380,7 +405,7 @@ class TestCOSESampleWithEncode:
         # The sender side:
         nonce = enc_key.generate_nonce()
         sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
-        encoded = sender.encode(b"Hello world!", enc_key, unprotected={5: nonce})
+        encoded = sender.encode(b"Hello world!", enc_key, unprotected={COSEHeaders.IV: nonce})
 
         # The recipient side:
         recipient = COSE.new()
@@ -399,8 +424,8 @@ class TestCOSESampleWithEncode:
         encoded3 = sender.encode(
             b"Hello world!",
             enc_key,
-            protected={1: 24},
-            unprotected={4: b"01", 5: nonce},
+            protected={COSEHeaders.ALG: COSEAlgs.CHACHA20_POLY1305},
+            unprotected={COSEHeaders.KID: b"01", COSEHeaders.IV: nonce},
         )
         assert b"Hello world!" == recipient.decode(encoded3, enc_key)
 
@@ -412,7 +437,7 @@ class TestCOSESampleWithEncode:
         # The sender side:
         nonce = enc_key.generate_nonce()
         sender = COSE.new(alg_auto_inclusion=True, kid_auto_inclusion=True)
-        encoded = sender.encode(b"Hello world!", enc_key, unprotected={5: nonce})
+        encoded = sender.encode(b"Hello world!", enc_key, unprotected={COSEHeaders.IV: nonce})
 
         # The notary side:
         notary = Signer.from_jwk(
@@ -465,16 +490,12 @@ class TestCOSESampleWithEncode:
             b"This is the content.",
             rpk,
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
+            external_aad=b"COSE-HPKE app",
         )
 
         # print(encoded.hex())
@@ -492,7 +513,7 @@ class TestCOSESampleWithEncode:
             }
         )
         recipient = COSE.new()
-        assert b"This is the content." == recipient.decode(encoded, rsk)
+        assert b"This is the content." == recipient.decode(encoded, rsk, external_aad=b"COSE-HPKE app")
 
     def test_cose_usage_examples_cose_encrypt(self):
         enc_key = COSEKey.generate_symmetric_key(alg="ChaCha20/Poly1305", kid="01")
@@ -515,12 +536,12 @@ class TestCOSESampleWithEncode:
         assert b"Hello world!" == recipient.decode(encoded, enc_key)
 
         # variation
-        r = Recipient.new(unprotected={1: -6})
+        r = Recipient.new(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT})
         encoded2 = sender.encode(
             b"Hello world!",
             enc_key,
-            protected={1: 24},  # ChaCha20/Poly1305
-            unprotected={4: enc_key.kid, 5: nonce},
+            protected={COSEHeaders.ALG: COSEAlgs.CHACHA20_POLY1305},
+            unprotected={COSEHeaders.KID: enc_key.kid, COSEHeaders.IV: nonce},
             recipients=[r],
         )
         assert b"Hello world!" == recipient.decode(encoded2, enc_key)
@@ -580,7 +601,7 @@ class TestCOSESampleWithEncode:
     def test_cose_usage_examples_cose_encrypt_hpke(self):
         # The sender side:
         enc_key = COSEKey.generate_symmetric_key(alg="A128GCM")
-        rpk = COSEKey.from_jwk(
+        rpk1 = COSEKey.from_jwk(
             {
                 "kty": "EC",
                 "kid": "01",
@@ -589,34 +610,47 @@ class TestCOSESampleWithEncode:
                 "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
             }
         )
-        r = Recipient.new(
+        r1 = Recipient.new(
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
-            recipient_key=rpk,
+            recipient_key=rpk1,
+        )
+        rpk2 = COSEKey.from_jwk(
+            {
+                "kty": "OKP",
+                "kid": "02",
+                "crv": "X25519",
+                "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
+            },
+        )
+        r2 = Recipient.new(
+            protected={
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_X25519_SHA256_CHACHA20POLY1305,
+            },
+            unprotected={
+                COSEHeaders.KID: b"02",
+            },
+            recipient_key=rpk2,
         )
         sender = COSE.new()
         encoded = sender.encode(
             b"This is the content.",
             enc_key,
             protected={
-                1: 1,  # alg: "A128GCM"
+                COSEHeaders.ALG: COSEAlgs.A128GCM,
             },
-            recipients=[r],
+            recipients=[r1, r2],
+            external_aad=b"COSE-HPKE app",
         )
 
         # print(encoded.hex())
 
         # The recipient side:
-        rsk = COSEKey.from_jwk(
+        rsk1 = COSEKey.from_jwk(
             {
                 "kty": "EC",
                 "kid": "01",
@@ -627,7 +661,18 @@ class TestCOSESampleWithEncode:
             }
         )
         recipient = COSE.new()
-        assert b"This is the content." == recipient.decode(encoded, rsk)
+        assert b"This is the content." == recipient.decode(encoded, rsk1, external_aad=b"COSE-HPKE app")
+
+        rsk2 = COSEKey.from_jwk(
+            {
+                "kty": "OKP",
+                "kid": "02",
+                "crv": "X25519",
+                "x": "y3wJq3uXPHeoCO4FubvTc7VcBuqpvUrSvU6ZMbHDTCI",
+                "d": "vsJ1oX5NNi0IGdwGldiac75r-Utmq3Jq4LGv48Q_Qc4",
+            },
+        )
+        assert b"This is the content." == recipient.decode(encoded, rsk2, external_aad=b"COSE-HPKE app")
 
     def test_cose_usage_examples_cose_encrypt_hpke_with_1st_layer_hpke(self):
         # The sender side:
@@ -643,15 +688,10 @@ class TestCOSESampleWithEncode:
         )
         r = Recipient.new(
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
             recipient_key=rpk,
         )
@@ -660,15 +700,10 @@ class TestCOSESampleWithEncode:
             sender.encode(
                 b"This is the content.",
                 protected={
-                    1: -1,  # alg: "HPKE"
+                    COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
                 },
                 unprotected={
-                    4: b"xx",  # kid: "xx"
-                    -4: [  # HPKE sender information
-                        0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                        0x0001,  # kdf: HKDF-SHA256
-                        0x0001,  # aead: AES-128-GCM
-                    ],
+                    COSEHeaders.KID: b"xx",
                 },
                 recipients=[r],
             )
@@ -689,15 +724,10 @@ class TestCOSESampleWithEncode:
         )
         r = Recipient.new(
             protected={
-                1: -1,  # alg: "HPKE"
+                COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
             },
             unprotected={
-                4: b"01",  # kid: "01"
-                -4: [  # HPKE sender information
-                    0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                    0x0001,  # kdf: HKDF-SHA256
-                    0x0001,  # aead: AES-128-GCM
-                ],
+                COSEHeaders.KID: b"01",
             },
             recipient_key=rpk,
         )
@@ -706,15 +736,10 @@ class TestCOSESampleWithEncode:
             sender.encode(
                 b"This is the content.",
                 protected={
-                    1: -1,  # alg: "HPKE"
+                    COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM,
                 },
                 unprotected={
-                    4: b"xx",  # kid: "xx"
-                    -4: {  # HPKE sender information
-                        1: 0x0010,  # kem: DHKEM(P-256, HKDF-SHA256)
-                        2: 0x0001,  # kdf: HKDF-SHA256
-                        3: 0x0001,  # aead: AES-128-GCM
-                    },
+                    COSEHeaders.KID: b"xx",
                 },
                 recipients=[r],
             )
@@ -939,11 +964,22 @@ class TestCOSESampleWithEncode:
         encoded3 = sender.encode(
             b"Hello world!",
             priv_key,
-            protected={1: -7},
-            unprotected={4: b"01"},
+            protected={COSEHeaders.ALG: COSEAlgs.ES256},
+            unprotected={COSEHeaders.KID: b"01"},
         )
         assert b"Hello world!" == recipient.decode(encoded3, pub_key)
 
+        # zero-length map protected header
+        encoded4 = sender.encode(
+            b"Hello world!",
+            priv_key,
+            unprotected={COSEHeaders.ALG: COSEAlgs.ES256, COSEHeaders.KID: b"01"},
+        )
+        loaded_encoded4 = loads(encoded4)
+        loaded_encoded4.value[0] = bytes.fromhex("a0")  # << {} >>
+        encoded4 = dumps(loaded_encoded4)
+        assert b"Hello world!" == recipient.decode(encoded4, pub_key)
+
     def test_cose_usage_examples_cose_signature1_countersignature(self):
         # The sender side:
         priv_key = COSEKey.from_jwk(
@@ -1031,24 +1067,34 @@ class TestCOSESampleWithEncode:
 
     def test_cose_usage_examples_cose_signature_countersignature(self):
         # The sender side:
-        signer = Signer.from_jwk(
+        s1 = Signer.from_jwk(
             {
                 "kty": "EC",
-                "kid": "01",
+                "kid": "s01",
                 "crv": "P-256",
                 "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
                 "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
                 "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
             },
         )
+        s2 = Signer.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "s02",
+                "crv": "P-384",
+                "x": "_XyN9woHaS0mPimSW-etwJMEDSzxIMjp4PjezavU8SHJoClz1bQrcmPb1ZJxHxhI",
+                "y": "GCNfc32p9sRotx7u2oDGJ3Eqz6q5zPHLdizNn83oRsUTN31eCWfGLHWRury3xF50",
+                "d": "1pImEKbrr771-RKi8Tb7tou_WjiR7kwui_nMu16449rk3lzAqf9buUhTkJ-pogkb",
+            },
+        )
         sender = COSE.new()
-        encoded = sender.encode(b"Hello world!", signers=[signer])
+        encoded = sender.encode(b"Hello world!", signers=[s1, s2])
 
         # The notary side:
         notary = Signer.from_jwk(
             {
-                "kid": "01",
                 "kty": "OKP",
+                "kid": "n01",
                 "crv": "Ed25519",
                 "alg": "EdDSA",
                 "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
@@ -1058,31 +1104,131 @@ class TestCOSESampleWithEncode:
         countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
 
         # The recipient side:
-        pub_key = COSEKey.from_jwk(
+        pk1 = COSEKey.from_jwk(
             {
                 "kty": "EC",
-                "kid": "01",
+                "kid": "s01",
                 "crv": "P-256",
                 "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
                 "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
             }
         )
-        notary_pub_key = COSEKey.from_jwk(
+        pk2 = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "s02",
+                "crv": "P-384",
+                "x": "_XyN9woHaS0mPimSW-etwJMEDSzxIMjp4PjezavU8SHJoClz1bQrcmPb1ZJxHxhI",
+                "y": "GCNfc32p9sRotx7u2oDGJ3Eqz6q5zPHLdizNn83oRsUTN31eCWfGLHWRury3xF50",
+            },
+        )
+        pkn = COSEKey.from_jwk(
             {
-                "kid": "01",
                 "kty": "OKP",
+                "kid": "n01",
                 "crv": "Ed25519",
                 "alg": "EdDSA",
                 "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
             },
         )
         recipient = COSE.new()
-        assert b"Hello world!" == recipient.decode(encoded, pub_key)
+        assert b"Hello world!" == recipient.decode(countersigned, pk1)
+        assert b"Hello world!" == recipient.decode(countersigned, pk2)
 
         try:
-            sig = COSEMessage.loads(countersigned).counterverify(notary_pub_key)
+            sig = COSEMessage.loads(countersigned).counterverify(pkn)
         except Exception as err:
             pytest.fail(f"failed to verify: {err}")
         countersignature = COSEMessage.from_cose_signature(sig)
-        assert countersignature.protected[1] == -8  # alg: "EdDSA"
-        assert countersignature.unprotected[4] == b"01"  # kid: b"01"
+        assert countersignature.protected[COSEHeaders.ALG] == COSEAlgs.EDDSA
+        assert countersignature.unprotected[COSEHeaders.KID] == b"n01"
+
+    def test_cose_usage_examples_detached_cose_sign_with_countersignature(self):
+        # The sender side:
+        s1 = Signer.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "s01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+            },
+        )
+        s2 = Signer.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "s02",
+                "crv": "P-384",
+                "x": "_XyN9woHaS0mPimSW-etwJMEDSzxIMjp4PjezavU8SHJoClz1bQrcmPb1ZJxHxhI",
+                "y": "GCNfc32p9sRotx7u2oDGJ3Eqz6q5zPHLdizNn83oRsUTN31eCWfGLHWRury3xF50",
+                "d": "1pImEKbrr771-RKi8Tb7tou_WjiR7kwui_nMu16449rk3lzAqf9buUhTkJ-pogkb",
+            },
+        )
+        sender = COSE.new()
+        encoded = sender.encode(b"Hello world!", signers=[s1, s2])
+
+        # The notary side:
+        notary = Signer.from_jwk(
+            {
+                "kty": "OKP",
+                "kid": "n01",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+            },
+        )
+        countersigned = COSEMessage.loads(encoded).countersign(notary).dumps()
+
+        msg, payload = COSEMessage.loads(countersigned).detach_payload()
+        # _, sig0 = COSEMessage.from_cose_signature(msg.signatures[0]).detach_payload()
+        # _, sig1 = COSEMessage.from_cose_signature(msg.signatures[1]).detach_payload()
+        detached = msg.dumps()
+
+        # print(detached.hex())
+
+        # The recipient side:
+        pk1 = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "s01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+            }
+        )
+        pk2 = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "s02",
+                "crv": "P-384",
+                "x": "_XyN9woHaS0mPimSW-etwJMEDSzxIMjp4PjezavU8SHJoClz1bQrcmPb1ZJxHxhI",
+                "y": "GCNfc32p9sRotx7u2oDGJ3Eqz6q5zPHLdizNn83oRsUTN31eCWfGLHWRury3xF50",
+            },
+        )
+        pkn = COSEKey.from_jwk(
+            {
+                "kty": "OKP",
+                "kid": "n01",
+                "crv": "Ed25519",
+                "alg": "EdDSA",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+            },
+        )
+
+        recipient = COSE.new()
+        # msg = COSEMessage.loads(detached)
+        # msg.signatures[0][2] = sig0
+        # msg.signatures[1][2] = sig1
+        # attached = msg.dumps()
+        assert b"Hello world!" == recipient.decode(detached, pk1, detached_payload=payload)
+        assert b"Hello world!" == recipient.decode(detached, pk2, detached_payload=payload)
+
+        try:
+            sig = COSEMessage.loads(countersigned).counterverify(pkn)
+        except Exception as err:
+            pytest.fail(f"failed to verify: {err}")
+        countersignature = COSEMessage.from_cose_signature(sig)
+        assert countersignature.protected[COSEHeaders.ALG] == COSEAlgs.EDDSA
+        assert countersignature.unprotected[COSEHeaders.KID] == b"n01"
