@@ -10,6 +10,7 @@ import cbor2
 import pytest
 
 from cwt import COSE, COSEKey, Recipient
+from cwt.enums import COSEAlgs, COSEHeaders
 from cwt.exceptions import DecodeError
 from cwt.recipient_interface import RecipientInterface
 from cwt.recipients import Recipients
@@ -130,10 +131,10 @@ class TestRecipientInterface:
         assert res[2] == b""
 
     def test_recipient_constructor_with_args(self):
-        child = RecipientInterface(unprotected={1: -6, 4: b"our-secret"})
+        child = RecipientInterface(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"})
         r = RecipientInterface(
             protected={"foo": "bar"},
-            unprotected={1: -1, 4: b"our-secret"},
+            unprotected={COSEHeaders.ALG: -1, COSEHeaders.KID: b"our-secret"},
             recipients=[child],
         )
         assert isinstance(r.protected, dict)
@@ -151,7 +152,7 @@ class TestRecipientInterface:
         assert len(res[3][0]) == 3
 
     def test_recipient_constructor_with_empty_recipients(self):
-        r = RecipientInterface(unprotected={1: -6, 4: b"our-secret"}, recipients=[])
+        r = RecipientInterface(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"}, recipients=[])
         assert isinstance(r, RecipientInterface)
         assert r.protected == {}
         assert isinstance(r.unprotected, dict)
@@ -161,9 +162,9 @@ class TestRecipientInterface:
         assert len(res) == 3
 
     def test_recipient_constructor_with_alg_a128kw(self):
-        r = RecipientInterface(protected={1: -3}, unprotected={4: b"our-secret"})
+        r = RecipientInterface(protected={COSEHeaders.ALG: COSEAlgs.A128KW}, unprotected={COSEHeaders.KID: b"our-secret"})
         assert isinstance(r, RecipientInterface)
-        assert r.alg == -3
+        assert r.alg == COSEAlgs.A128KW
         assert isinstance(r.protected, dict)
         assert isinstance(r.unprotected, dict)
         assert r.ciphertext == b""
@@ -172,9 +173,12 @@ class TestRecipientInterface:
         assert len(res) == 3
 
     def test_recipient_constructor_with_alg_a128kw_with_iv(self):
-        r = RecipientInterface(protected={1: -3}, unprotected={4: b"our-secret", 5: b"aabbccddee"})
+        r = RecipientInterface(
+            protected={COSEHeaders.ALG: COSEAlgs.A128KW},
+            unprotected={COSEHeaders.KID: b"our-secret", COSEHeaders.IV: b"aabbccddee"},
+        )
         assert isinstance(r, RecipientInterface)
-        assert r.alg == -3
+        assert r.alg == COSEAlgs.A128KW
         assert isinstance(r.protected, dict)
         assert isinstance(r.unprotected, dict)
         assert r.ciphertext == b""
@@ -187,56 +191,56 @@ class TestRecipientInterface:
         [
             (
                 {"foo": "bar"},
-                {1: -6, 4: b"our-secret"},
+                {COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"},
                 b"",
                 [],
                 "protected header should be empty.",
             ),
             (
                 {},
-                {1: -6, 4: b"our-secret"},
+                {COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"},
                 b"xxx",
                 [],
                 "ciphertext should be zero-length bytes.",
             ),
             (
                 {},
-                {1: -6, 4: b"our-secret"},
+                {COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"},
                 b"",
                 [RecipientInterface()],
                 "recipients should be absent.",
             ),
             (
                 {},
-                {4: "our-secret"},
+                {COSEHeaders.KID: "our-secret"},
                 b"",
                 [RecipientInterface()],
                 "unprotected[4](kid) should be bytes.",
             ),
             (
-                {4: "our-secret"},
+                {COSEHeaders.KID: "our-secret"},
                 {},
                 b"",
                 [RecipientInterface()],
                 "protected[4](kid) should be bytes.",
             ),
             (
-                {1: "alg-a"},
-                {4: b"our-secret"},
+                {COSEHeaders.ALG: "alg-a"},
+                {COSEHeaders.KID: b"our-secret"},
                 b"",
                 [RecipientInterface()],
                 "protected[1](alg) should be int.",
             ),
             (
                 {},
-                {1: "alg-a", 4: b"our-secret"},
+                {COSEHeaders.ALG: "alg-a", COSEHeaders.KID: b"our-secret"},
                 b"",
                 [RecipientInterface()],
                 "unprotected[1](alg) should be int.",
             ),
             (
                 {},
-                {4: b"our-secret", 5: "xxx"},
+                {COSEHeaders.KID: b"our-secret", COSEHeaders.IV: "xxx"},
                 b"",
                 [RecipientInterface()],
                 "unprotected[5](iv) should be bytes.",
@@ -252,7 +256,7 @@ class TestRecipientInterface:
     def test_recipient_constructor_with_invalid_recipients(self):
         child = {}
         with pytest.raises(ValueError) as err:
-            RecipientInterface(unprotected={1: 0, 4: b"our-secret"}, recipients=[child])
+            RecipientInterface(unprotected={COSEHeaders.ALG: 0, COSEHeaders.KID: b"our-secret"}, recipients=[child])
             pytest.fail("RecipientInterface() should fail.")
         assert "Invalid child recipient." in str(err.value)
 
@@ -271,12 +275,12 @@ class TestRecipient:
                 "alg should be specified.",
             ),
             (
-                {1: -6},
-                {1: -6},
+                {COSEHeaders.ALG: COSEAlgs.DIRECT},
+                {COSEHeaders.ALG: COSEAlgs.DIRECT},
                 "alg appear both in protected and unprotected.",
             ),
             (
-                {1: -65535},
+                {COSEHeaders.ALG: -65535},
                 {},
                 # "Unsupported or unknown alg(1): -65535.",
                 "context should be set.",
@@ -289,22 +293,38 @@ class TestRecipient:
             pytest.fail("Recipient() should fail.")
         assert msg in str(err.value)
 
+    def test_recipient_new_does_not_modify_protected(self, rpk1, rsk1):
+        protected = {}
+        unprotected = {"alg": "ECDH-ES+A128KW"}
+        kdf_context = {
+            "alg": "A128KW",
+            "supp_pub": {
+                "key_data_length": 128,
+                "protected": protected,
+                "other": "test",
+            },
+        }
+        _ = Recipient.new(
+            protected=protected, unprotected=unprotected, sender_key=rsk1, recipient_key=rpk1, context=kdf_context
+        )
+        assert protected == {}
+
     def test_recipient_from_jwk_with_str(self):
         recipient = Recipient.new(unprotected={"alg": "direct"})
         assert isinstance(recipient, RecipientInterface)
-        assert recipient.alg == -6
+        assert recipient.alg == COSEAlgs.DIRECT
 
     def test_recipient_from_jwk_with_dict(self):
         k = COSEKey.from_jwk({"kty": "oct", "alg": "A128KW", "key_ops": ["wrapKey"]})
         recipient = Recipient.new(unprotected={"alg": "A128KW"}, sender_key=k)
         assert isinstance(recipient, RecipientInterface)
-        assert recipient.alg == -3
+        assert recipient.alg == COSEAlgs.A128KW
 
     def test_recipient_from_jwk_with_dict_and_with_byte_formatted_kid(self):
         k = COSEKey.from_jwk({"kty": "oct", "kid": b"01", "alg": "A128KW", "key_ops": ["wrapKey"]})
         recipient = Recipient.new(unprotected={"kid": b"01", "alg": "A128KW"}, sender_key=k)
         assert isinstance(recipient, RecipientInterface)
-        assert recipient.alg == -3
+        assert recipient.alg == COSEAlgs.A128KW
         assert recipient.kid == b"01"
 
     # def test_recipient_from_jwk_with_context(self):
@@ -327,7 +347,7 @@ class TestRecipient:
     #         }
     #     )
     #     assert isinstance(recipient, RecipientInterface)
-    #     assert recipient.alg == -10
+    #     assert recipient.alg == COSEAlgs.DIRECT_HKDF_SHA256
     #     assert recipient._unprotected[-21] == b"sender-01"
     #     assert recipient._unprotected[-22] == b"xxx"
     #     assert recipient._unprotected[-23] == b"yyy"
@@ -351,7 +371,7 @@ class TestRecipient:
     #         }
     #     )
     #     assert isinstance(recipient, RecipientInterface)
-    #     assert recipient.alg == -10
+    #     assert recipient.alg == COSEAlgs.DIRECT_HKDF_SHA256
     #     assert recipient._unprotected[-21] == b"sender-01"
     #     assert -22 not in recipient._unprotected
     #     assert -23 not in recipient._unprotected
@@ -375,7 +395,7 @@ class TestRecipient:
     #         }
     #     )
     #     assert isinstance(recipient, RecipientInterface)
-    #     assert recipient.alg == -10
+    #     assert recipient.alg == COSEAlgs.DIRECT_HKDF_SHA256
     #     assert -21 not in recipient._unprotected
     #     assert recipient._unprotected[-22] == b"xxx"
     #     assert -23 not in recipient._unprotected
@@ -399,7 +419,7 @@ class TestRecipient:
     #         }
     #     )
     #     assert isinstance(recipient, RecipientInterface)
-    #     assert recipient.alg == -10
+    #     assert recipient.alg == COSEAlgs.DIRECT_HKDF_SHA256
     #     assert -21 not in recipient._unprotected
     #     assert -22 not in recipient._unprotected
     #     assert recipient._unprotected[-23] == b"yyy"
@@ -413,7 +433,7 @@ class TestRecipient:
             (
                 {"foo": "bar"},
                 {"kty": "oct", "foo": "bar"},
-                "alg(3) should be int or str(tstr).",
+                "alg(3) not found.",
             ),
             (
                 {"alg": "xxx"},
@@ -463,7 +483,7 @@ class TestRecipient:
             (
                 {"alg": "direct+HKDF-SHA-256", "context": []},
                 {"kty": "oct", "alg": "direct+HKDF-SHA-256"},
-                "Unsupported or unknown alg(3): -10.",
+                f"Unsupported or unknown alg(3): {COSEAlgs.DIRECT_HKDF_SHA256}.",
             ),
         ],
     )
@@ -485,14 +505,14 @@ class TestRecipients:
 
     def test_recipients_constructor_with_recipient_alg_direct(self):
         key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64", kid="our-secret")
-        r = Recipients([Recipient.new(unprotected={1: -6, 4: b"our-secret"})])
+        r = Recipients([Recipient.new(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"})])
         key = r.derive_key([key], key.alg, b"", b"")
         assert key.kty == 4
-        assert key.alg == 4
+        assert key.alg == COSEAlgs.HS256_64
         assert key.kid == b"our-secret"
 
     def test_recipients_derive_key_without_key(self):
-        r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"our-secret"})])
+        r = Recipients([RecipientInterface(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"})])
         with pytest.raises(ValueError) as err:
             r.derive_key([], 0, b"", b"")
             pytest.fail("derive_key() should fail.")
@@ -535,7 +555,7 @@ class TestRecipients:
     #     )
     #     rs = Recipients([r1, r2])
     #     key = rs.derive_key(context=context, keys=[material])
-    #     assert key.alg == 10
+    #     assert key.alg == COSEAlgs.AES_CCM_16_64_128
     #     assert key.kid == b"02"
 
     # def test_recipients_derive_key_with_multiple_keys(self, material):
@@ -576,13 +596,13 @@ class TestRecipients:
     #         },
     #     )
     #     rs = Recipients([r1, r2, r3])
-    #     key = rs.derive_key(keys=[k3], alg=7)
-    #     assert key.alg == 7
+    #     key = rs.derive_key(keys=[k3], alg=COSEAlgs.HS512)
+    #     assert key.alg == COSEAlgs.HS512
     #     assert key.kid == b"03"
 
     def test_recipients_derive_key_with_different_kid(self):
         key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64", kid="our-secret")
-        r = Recipients([RecipientInterface(unprotected={1: -6, 4: b"your-secret"})])
+        r = Recipients([RecipientInterface(unprotected={COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"your-secret"})])
         with pytest.raises(ValueError) as err:
             r.derive_key([key], key.alg, b"", b"")
             pytest.fail("derive_key() should fail.")
@@ -590,13 +610,19 @@ class TestRecipients:
 
     def test_recipients_from_list(self):
         try:
-            Recipients.from_list([[cbor2.dumps({1: -10}), {-20: b"aabbccddeefff"}, b""]], context={"alg": "A128GCM"})
+            Recipients.from_list(
+                [[cbor2.dumps({COSEHeaders.ALG: COSEAlgs.DIRECT_HKDF_SHA256}), {-20: b"aabbccddeefff"}, b""]],
+                context={"alg": "A128GCM"},
+            )
         except Exception:
             pytest.fail("from_list() should not fail.")
 
     def test_recipients_from_list_with_empty_recipients(self):
         try:
-            Recipients.from_list([[cbor2.dumps({1: -10}), {-20: b"aabbccddeefff"}, b"", []]], context={"alg": "A128GCM"})
+            Recipients.from_list(
+                [[cbor2.dumps({COSEHeaders.ALG: COSEAlgs.DIRECT_HKDF_SHA256}), {-20: b"aabbccddeefff"}, b"", []]],
+                context={"alg": "A128GCM"},
+            )
         except Exception:
             pytest.fail("from_list() should not fail.")
 
@@ -605,10 +631,10 @@ class TestRecipients:
             Recipients.from_list(
                 [
                     [
-                        cbor2.dumps({1: -10}),
+                        cbor2.dumps({COSEHeaders.ALG: COSEAlgs.DIRECT_HKDF_SHA256}),
                         {-20: b"aabbccddeefff"},
                         b"",
-                        [[b"", {1: -6, 4: b"our-secret"}, b""]],
+                        [[b"", {COSEHeaders.ALG: COSEAlgs.DIRECT, COSEHeaders.KID: b"our-secret"}, b""]],
                     ]
                 ],
                 context={"alg": "A128GCM"},
@@ -649,7 +675,9 @@ class TestRecipients:
         assert msg in str(err.value)
 
     def test_recipients_open_without_key(self):
-        r = RecipientInterface(protected={1: -1}, unprotected={4: b"01", -4: [0x0010, 0x0001, 0x0001]})
+        r = RecipientInterface(
+            protected={COSEHeaders.ALG: -1}, unprotected={COSEHeaders.KID: b"01", -4: [0x0010, 0x0001, 0x0001]}
+        )
         rs = Recipients([r])
         with pytest.raises(ValueError) as err:
             rs.derive_key([], 0, b"", b"")
@@ -674,14 +702,14 @@ class TestRecipients:
                 "y": "BGU5soLgsu_y7GN2I3EPUXS9EZ7Sw0qif-V70JtInFI",
             }
         )
-        r = Recipient.new(protected={1: -1}, unprotected={-4: [0x0010, 0x0001, 0x0001]}, recipient_key=rpk)
+        r = Recipient.new(protected={COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM}, recipient_key=rpk)
         r.encode(enc_key.key)
         sender = COSE.new()
         encoded = sender.encode_and_encrypt(
             b"This is the content.",
             enc_key,
             protected={
-                1: 1,  # alg: "A128GCM"
+                COSEHeaders.ALG: COSEAlgs.A128GCM,  # alg: "A128GCM"
             },
             recipients=[r],
         )
@@ -699,7 +727,7 @@ class TestRecipients:
                 "y": "BGU5soLgsu_y7GN2I3EPUXS9EZ7Sw0qif-V70JtInFI",
             }
         )
-        r = Recipient.new(protected={1: -1}, unprotected={-4: [0x0010, 0x0001, 0x0001]}, recipient_key=rpk)
+        r = Recipient.new(protected={COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM}, recipient_key=rpk)
         sender = COSE.new()
         encoded = sender.encode_and_encrypt(
             b"This is the content.",
@@ -724,7 +752,7 @@ class TestRecipients:
                 "y": "BGU5soLgsu_y7GN2I3EPUXS9EZ7Sw0qif-V70JtInFI",
             }
         )
-        r = Recipient.new(protected={1: -1}, unprotected={-4: [0x0010, 0x0001, 0x0001]}, recipient_key=rpk)
+        r = Recipient.new(protected={COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM}, recipient_key=rpk)
         r.encode(enc_key.key)
         sender = COSE.new()
         encoded = sender.encode_and_encrypt(
@@ -741,14 +769,18 @@ class TestRecipients:
 
     def test_recipients_open_with_multiple_rsks(self, rpk2, rsk1, rsk2):
         enc_key = COSEKey.from_symmetric_key(alg="A128GCM")
-        r = Recipient.new(protected={1: -1}, unprotected={4: b"02", -4: [0x0010, 0x0001, 0x0001]}, recipient_key=rpk2)
+        r = Recipient.new(
+            protected={COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM},
+            unprotected={COSEHeaders.KID: b"02"},
+            recipient_key=rpk2,
+        )
         r.encode(enc_key.key)
         sender = COSE.new(alg_auto_inclusion=True)
         encoded = sender.encode_and_encrypt(
             b"This is the content.",
             key=enc_key,
             # protected={
-            #     1: -1,  # alg: "HPKE"
+            #     COSEHeaders.ALG: -1,  # alg: "HPKE"
             # },
             recipients=[r],
         )
@@ -757,7 +789,11 @@ class TestRecipients:
 
     def test_recipients_open_with_invalid_rsk(self, rpk1):
         enc_key = COSEKey.from_symmetric_key(alg="A128GCM")
-        r = Recipient.new(protected={1: -1}, unprotected={4: b"02", -4: [0x0010, 0x0001, 0x0001]}, recipient_key=rpk1)
+        r = Recipient.new(
+            protected={COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM},
+            unprotected={COSEHeaders.KID: b"02"},
+            recipient_key=rpk1,
+        )
         # r.encode(enc_key.to_bytes())
         sender = COSE.new()
         encoded = sender.encode_and_encrypt(
@@ -781,3 +817,177 @@ class TestRecipients:
             recipient.decode(encoded, [invalid_rsk])
             pytest.fail("decode() should fail.")
         assert "Failed to open." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "kw_alg, enc_alg",
+        [
+            ("A128KW", "A128CTR"),
+            ("A192KW", "A192CTR"),
+            ("A256KW", "A256CTR"),
+            ("A128KW", "A128CBC"),
+            ("A192KW", "A192CBC"),
+            ("A256KW", "A256CBC"),
+        ],
+    )
+    def test_recipients_aes(self, kw_alg, enc_alg):
+        kw_key = COSEKey.from_symmetric_key(alg=kw_alg)
+        enc_key = COSEKey.from_symmetric_key(alg=enc_alg)
+
+        # The sender side (must fail):
+        r = Recipient.new(unprotected={"alg": kw_alg}, sender_key=kw_key)
+        sender = COSE.new(alg_auto_inclusion=True)
+        with pytest.raises(ValueError) as err:
+            encoded = sender.encode_and_encrypt(
+                b"Hello world!",
+                enc_key,
+                recipients=[r],
+                enable_non_aead=False,
+            )
+            pytest.fail("encode_and_encrypt() should fail.")
+        assert "Deprecated non-AEAD algorithm" in str(err.value)
+
+        # The sender side (must fail):
+        with pytest.raises(ValueError) as err:
+            r = Recipient.new(protected={"alg": kw_alg}, sender_key=kw_key)
+            pytest.fail("encode_and_encrypt() should fail.")
+        assert "The protected header must be a zero-length string in key wrap mode with an AE algorithm." in str(err.value)
+
+        # The sender side (must fail):
+        r = Recipient.new(unprotected={"alg": kw_alg}, sender_key=kw_key)
+        sender = COSE.new(alg_auto_inclusion=True)
+        with pytest.raises(ValueError) as err:
+            encoded = sender.encode_and_encrypt(
+                b"Hello world!",
+                enc_key,
+                protected={"kid": "actually-not-protected"},
+                recipients=[r],
+                enable_non_aead=True,
+            )
+            pytest.fail("encode_and_encrypt() should fail.")
+        assert "protected header MUST be zero-length" in str(err.value)
+
+        # The sender side:
+        r = Recipient.new(unprotected={"alg": kw_alg}, sender_key=kw_key)
+        sender = COSE.new(alg_auto_inclusion=True)
+        encoded = sender.encode_and_encrypt(
+            b"Hello world!",
+            enc_key,
+            recipients=[r],
+            enable_non_aead=True,
+        )
+
+        # The recipient side (must fail):
+        recipient = COSE.new()
+        with pytest.raises(ValueError) as err:
+            _ = recipient.decode(encoded, keys=[kw_key])  # the option enable_non_aead=False by default
+            pytest.fail("decode() should fail for non-AEAD without enable_non_aead=True.")
+        assert f"Deprecated non-AEAD algorithm: {enc_key._alg}." == str(err.value)
+
+        # The recipient side:
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, keys=[kw_key], enable_non_aead=True)
+
+    @pytest.mark.parametrize(
+        "enc_alg",
+        [
+            ("A128CTR"),
+            ("A192CTR"),
+            ("A256CTR"),
+            ("A128CBC"),
+            ("A192CBC"),
+            ("A256CBC"),
+        ],
+    )
+    def test_recipients_hpke(self, rsk1, rsk2, enc_alg):
+        enc_key = COSEKey.from_symmetric_key(alg=enc_alg)
+        rpk = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                # "kid": "02",
+                "crv": "P-256",
+                "x": "-eZXC6nV-xgthy8zZMCN8pcYSeE2XfWWqckA2fsxHPc",
+                "y": "BGU5soLgsu_y7GN2I3EPUXS9EZ7Sw0qif-V70JtInFI",
+            }
+        )
+        r = Recipient.new(unprotected={COSEHeaders.ALG: COSEAlgs.HPKE_BASE_P256_SHA256_AES128GCM}, recipient_key=rpk)
+        r.encode(enc_key.key)
+        sender = COSE.new()
+        encoded = sender.encode_and_encrypt(
+            b"This is the content.",
+            enc_key,
+            unprotected={"alg": enc_alg},
+            recipients=[r],
+            enable_non_aead=True,
+        )
+        recipient = COSE.new()
+        assert b"This is the content." == recipient.decode(encoded, [rsk1, rsk2], enable_non_aead=True)
+
+    @pytest.mark.parametrize(
+        "key_agreement_alg, key_agreement_alg_id, kw_alg, enc_alg",
+        [
+            ("ECDH-ES+A128KW", COSEAlgs.ECDH_ES_A128KW, "A128KW", "A128CTR"),
+            ("ECDH-ES+A192KW", COSEAlgs.ECDH_ES_A192KW, "A192KW", "A192CTR"),
+            ("ECDH-ES+A256KW", COSEAlgs.ECDH_ES_A256KW, "A256KW", "A256CTR"),
+            ("ECDH-ES+A128KW", COSEAlgs.ECDH_ES_A128KW, "A128KW", "A128CBC"),
+            ("ECDH-ES+A192KW", COSEAlgs.ECDH_ES_A192KW, "A192KW", "A192CBC"),
+            ("ECDH-ES+A256KW", COSEAlgs.ECDH_ES_A256KW, "A256KW", "A256CBC"),
+        ],
+    )
+    def test_recipients_ecdh_es(self, key_agreement_alg, key_agreement_alg_id, kw_alg, enc_alg):
+        enc_key = COSEKey.from_symmetric_key(alg=enc_alg)
+        context = {
+            "alg": kw_alg,
+            "supp_pub": {
+                "key_data_length": len(enc_key.key) * 8,
+                "protected": {},
+            },
+        }
+
+        # The sender side:
+        rsk1 = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "01",
+                "crv": "P-256",
+                "x": "usWxHK2PmfnHKwXPS54m0kTcGJ90UiglWiGahtagnv8",
+                "y": "IBOL-C3BttVivg-lSreASjpkttcsz-1rb7btKLv8EX4",
+                "d": "V8kgd2ZBRuh2dgyVINBUqpPDr7BOMGcF22CQMIUHtNM",
+                "alg": key_agreement_alg,
+            }
+        )
+        rpk2 = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                # "kid": "02",
+                "crv": "P-256",
+                "x": "-eZXC6nV-xgthy8zZMCN8pcYSeE2XfWWqckA2fsxHPc",
+                "y": "BGU5soLgsu_y7GN2I3EPUXS9EZ7Sw0qif-V70JtInFI",
+            }
+        )
+        r = Recipient.new(unprotected={"alg": key_agreement_alg}, sender_key=rsk1, recipient_key=rpk2, context=context)
+
+        nonce = enc_key.generate_nonce()
+        sender = COSE.new()
+        encoded = sender.encode(
+            b"Hello world!",
+            enc_key,
+            protected={},
+            unprotected={"alg": enc_alg, "iv": nonce},
+            recipients=[r],
+            enable_non_aead=True,
+        )
+
+        # The recipient side:
+        rsk2 = COSEKey.from_jwk(
+            {
+                "kty": "EC",
+                "kid": "02",
+                "crv": "P-256",
+                "x": "-eZXC6nV-xgthy8zZMCN8pcYSeE2XfWWqckA2fsxHPc",
+                "y": "BGU5soLgsu_y7GN2I3EPUXS9EZ7Sw0qif-V70JtInFI",
+                "d": "kwibx3gas6Kz1V2fyQHKSnr-ybflddSjN0eOnbmLmyo",
+                "alg": key_agreement_alg,
+            }
+        )
+        recipient = COSE.new()
+        assert b"Hello world!" == recipient.decode(encoded, rsk2, context, enable_non_aead=True)
